/**
   Shade fragments according to their location
   in the chosen coordinate system.

   This class is a "pipeline stage" because it
   implements the PipelineStage interface, which
   consistes of a single method, processModel().
*/
import java.awt.Color;

public class ColorFieldShader implements PipelineStage
{
   private ColorFunctor f;
   private int coordSystem; // use 0 for model, 1 for world, 2 for view

   public ColorFieldShader(ColorFunctor f, int coord)
   {
      this.f = f;
      this.coordSystem = coord;
   }

   public void setCoordSystem(int coord)
   {
      this.coordSystem = coord;
   }

   /**
      Color the model using the color field determined
      by the given "color functions".
   */
   public Model processModel(Model model)
   {
      for (Triangle t : model.triangleList)
      {
         // Walk this triangle's list of fragments.
         for (Fragment frag : t.fragments)
         {
            // Get this fragment's barycentric coordinates.
            double alpha = frag.alpha;
            double beta  = frag.beta;
            double gamma = frag.gamma;

            // Get the coordinates of each vertex in the
            // triangle (use either model coordinates,
            // world coordinates, or view coordinates.)
            double x0 = t.v[0].previous[coordSystem].x;
            double x1 = t.v[1].previous[coordSystem].x;
            double x2 = t.v[2].previous[coordSystem].x;
            double y0 = t.v[0].previous[coordSystem].y;
            double y1 = t.v[1].previous[coordSystem].y;
            double y2 = t.v[2].previous[coordSystem].y;
            double z0 = t.v[0].previous[coordSystem].z;
            double z1 = t.v[1].previous[coordSystem].z;
            double z2 = t.v[2].previous[coordSystem].z;

            // Interpolate coordinate data from the triangle's
            // vertices to this fragment.
            double frag_x = alpha*x0 + beta*x1 + gamma*x2;
            double frag_y = alpha*y0 + beta*y1 + gamma*y2;
            double frag_z = alpha*z0 + beta*z1 + gamma*z2;

            // Color the fragment.
            frag.r = f.r(frag_x, frag_y, frag_z);
            frag.g = f.g(frag_x, frag_y, frag_z);
            frag.b = f.b(frag_x, frag_y, frag_z);
            frag.a = 0.0;
         }
      }
      return model;
   }
}