/*
   This program is a client of the ColorFieldShader
   pipeline stage. This program puts the ColorFieldShader
   pipeline stage into a rendering pipeline in place of the
   regular fragment shader stage.
*/
import java.io.File;
import java.awt.Color;

public class ColorFieldShaderClient
{
   public static void main(String[] args)
   {
      // Create several Model objects.
      Model m1 = new ObjSimpleModel(new File("resources/horse.obj"));
      m1.doBackFaceCulling = true;
      m1.modelTranslate(-2, 0, 0);
      m1.modelScale(0.5, 0.5, 0.5);

      Model m2 = new SphereModel(30, 30, 1);
      m2.doBackFaceCulling = true;
      m2.modelTranslate(3, 0, 0);
      m2.modelScale(2, 2, 2);

      Model m3 = new ObjSimpleModel(new File("resources/apple.obj"), true);
      m3.doBackFaceCulling = true;
      m3.modelTranslate(0, 3, 0);
      m3.modelScale(2, 2, 2);

      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the models to the Scene.
      scene.modelList.add(m1);
      scene.modelList.add(m2);
      scene.modelList.add(m3);

      // Give the Scene a view matrix.
      scene.view2Identity();

      // Give the scene a projection matrix.
      scene.proj2Identity();

      // Set up the scene's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      scene.projPerspective(fov, aspect, near, far);

      // Set up the scene's "camera".
      double eyex=0,    eyey=0,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      scene.viewLookAt(eyex,    eyey,    eyez,
                       centerx, centery, centerz,
                       upx,     upy,     upz);

      // Create a framebuffer to render our scene into.
      int vp_width  = 1200;
      int vp_height =  400;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.darkGray);

      ColorFunctor f = new
         ColorFunctor()
         {
            public double r(double x, double y, double z)
            {
               return Math.abs( Math.sin( x*x + y*y + z*z ) );
            }
            public double g(double x, double y, double z)
            {
               return Math.abs( Math.sin( (x-2)*(x-2) + (y-2)*(y-2) + (z-2)*(z-2) ) );
            }
            public double b(double x, double y, double z)
            {
               return Math.abs( Math.sin(z) );
            }
         };

      ColorFunctor f2 = new
         ColorFunctor()
         {
            public double r(double x, double y, double z)
            {
               return Math.abs( Math.sin( y ) );
            }
            public double g(double x, double y, double z)
            {
               return Math.abs( Math.cos( y ) );
            }
            public double b(double x, double y, double z)
            {
               return Math.abs( y/(1 + Math.abs(y)) );
            }
         };

      // Create a rendering pipeline,
      SceneRender renderer = new SceneRender(scene, fb);  // default pipeline
      // Create an instance of the shader pipeline stage.
      ColorFieldShader shader = new ColorFieldShader(f, 0);
      // Add this pipeline stage in place of the default fragment shader
      renderer.replacePipelineStage(11, shader);
      // Remove unneeded pipeline stages
      renderer.removePipelineStage(12); // remove texture stage
      renderer.removePipelineStage(5);  // remove both the lighting stages
      renderer.removePipelineStage(4);
      // Check the order of the pipeline stages            .
      System.err.println( renderer );


      // Set a viewport within the framebuffer.
      fb.setViewport(0, 0, 400, 400); //upper-left-hand-corner, width, height
      // Give the viewport a nice background color.
      fb.clearVP(java.awt.Color.darkGray.brighter().brighter());
      // Render our scene into the current viewport (using model coordinates).
      shader.setCoordSystem(Vertex.MODEL);
      renderer.renderScene();

      // Set a viewport within the framebuffer.
      fb.setViewport(400, 0, 400, 400); //upper-left-hand-corner, width, height
      // Give the viewport a nice background color.
      fb.clearVP(java.awt.Color.darkGray.brighter().brighter());
      // Render our scene into the current viewport (using world coordinates).
      shader.setCoordSystem(Vertex.WORLD);
      renderer.renderScene();

      // Set a viewport within the framebuffer.
      fb.setViewport(800, 0, 400, 400); //upper-left-hand-corner, width, height
      // Give the viewport a nice background color.
      fb.clearVP(java.awt.Color.darkGray.brighter().brighter());
      // Render our scene into the current viewport (using view coordinates).
      shader.setCoordSystem(Vertex.VIEW);
      renderer.renderScene();

      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("PPM_ColorFieldShaderClient_Frame%02d.ppm", 0) );

      // Spin the scene 360 degrees.
      for (int i = 1; i <= 36; i++)
      {
         // Rotate the new scene by 10 degrees.
         scene.viewRotate(10, 0, 1, 0);
         // Render again.
         // Set a viewport within the framebuffer.
         fb.setViewport(0, 0, 400, 400); //upper-left-hand-corner, width, height
         // Give the viewport a nice background color.
         fb.clearVP(java.awt.Color.darkGray.brighter().brighter());
         // Render our scene into the current viewport (using model coordinates).
         shader.setCoordSystem(Vertex.MODEL);
         renderer.renderScene();

         // Set a viewport within the framebuffer.
         fb.setViewport(400, 0, 400, 400); //upper-left-hand-corner, width, height
         // Give the viewport a nice background color.
         fb.clearVP(java.awt.Color.darkGray.brighter().brighter());
         // Render our scene into the current viewport (using world coordinates).
         shader.setCoordSystem(Vertex.WORLD);
         renderer.renderScene();

         // Set a viewport within the framebuffer.
         fb.setViewport(800, 0, 400, 400); //upper-left-hand-corner, width, height
         // Give the viewport a nice background color.
         fb.clearVP(java.awt.Color.darkGray.brighter().brighter());
         // Render our scene into the current viewport (using view coordinates).
         shader.setCoordSystem(Vertex.VIEW);
         renderer.renderScene();

         // Save the resulting image in a file.
         fb.dumpFB2File(String.format("PPM_ColorFieldShaderClient_Frame%02d.ppm", i));
      }
   }
}