/**
   Shade fragments according to their location
   in the chosen coordinate system.

   This class is a "pipeline stage" because it
   implements the PipelineStage interface, which
   consistes of a single method, processModel().
*/
import java.awt.Color;

public class ColorFragmentShader implements PipelineStage
{
   private Color c1;
   private Color c2;
   private double spacing;
   private int coordSystem; // use 0 for model, 1 for world, 2 for view

   public ColorFragmentShader(Color c1, Color c2, double spacing, int coord)
   {
      this.c1 = c1;
      this.c2 = c2;
      this.spacing = spacing;
      this.coordSystem = coord;
   }

   public void setCoordSystem(int coord)
   {
      this.coordSystem = coord;
   }

   /**
      Stripe the model with the colors c1 and c2 in the x-direction.
      This can be done in either model coordinates, world coordinates,
      or view coordinates.
   */
   public Model processModel(Model model)
   {
      for (Triangle t : model.triangleList)
      {
         // Walk this triangle's list of fragments.
         for (Fragment frag : t.fragments)
         {
            // Get this fragment's barycentric coordinates.
            double alpha = frag.alpha;
            double beta  = frag.beta;
            double gamma = frag.gamma;

            // Get the x-coordinate of each vertex in the
            // triangle (use either model coordinates,
            // world coordinates, or view coordinates.)
            double x0 = t.v[0].previous[coordSystem].x;
            double x1 = t.v[1].previous[coordSystem].x;
            double x2 = t.v[2].previous[coordSystem].x;

            // Interpolate coordinate data from the triangle's
            // vertices to this fragment.
            double frag_x = alpha*x0 + beta*x1 + gamma*x2;

            // Color the fragment.
            if ( 0 == (Math.floor(frag_x / spacing) % 2) )
            {
               frag.r =   c1.getRed() / 255.0;
               frag.g = c1.getGreen() / 255.0;
               frag.b =  c1.getBlue() / 255.0;
               frag.a = 0.0;
            }
            else
            {
               frag.r =   c2.getRed() / 255.0;
               frag.g = c2.getGreen() / 255.0;
               frag.b =  c2.getBlue() / 255.0;
               frag.a = 0.0;
            }
         }
      }
      return model;
   }
}